package org.processmining.indatabasepreprocessing.algorithms;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Collection;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.indatabasepreprocessing.parameters.DBConnectionParameter;
import org.processmining.socialnetwork.models.SocialNetwork;
import org.processmining.socialnetwork.models.SocialNetworkVisualization;
import org.processmining.socialnetwork.models.SocialNetworkVisualizationAlgorithm;
import org.processmining.socialnetwork.models.impl.TObjectDoubleHashMapSocialNetworkImpl;

public class InDBPreprocessingBooleanHoW implements InDBPreprocessingAlgorithm {
	private DBConnectionParameter parameter;
	private Statement stat;
	private SocialNetwork<String> socialnetwork;
	private SocialNetworkVisualizationAlgorithm<String> visualizationAlgorithm;
	private int divisor;
	private double maxValue;
	private String initializeType;
	
	public InDBPreprocessingBooleanHoW(UIPluginContext context, Connection conn, DBConnectionParameter parameter) {
		this.parameter = parameter;
		try {
			stat = conn.createStatement();
			stat.setFetchSize(100000);
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}

	public JComponent visualize() {
		long start = System.nanoTime();
		
		// retrieve and mine
		long startMining = System.nanoTime();
		countDivisor();
		mine();
		System.out.print("Boolean," + String.format("%.3f", (System.nanoTime() - startMining) / 1000.0) + ";" );
		
		// visualize
		long startVisualization = System.nanoTime();
		visualizationAlgorithm = SocialNetworkVisualizationAlgorithm.Factory
												.newInstance(SocialNetworkVisualizationAlgorithm.Algorithm.SIMPLE, SocialNetworkVisualization.Library.GRAPH_VIZ);
		visualizationAlgorithm.clear();
		visualizationAlgorithm.processNewNetwork(getNetwork(1.0)); // parameter need to have threshold
		JComponent component = visualizationAlgorithm.visualize();
		System.out.print(String.format("%.3f", (System.nanoTime() - startVisualization) / 1000.0) + ";" );
		
		System.out.print(String.format("%.3f", (System.nanoTime() - start) / 1000.0) + "\n" );
		
		return component;
	}
	
	private void mine() {
		socialnetwork = new TObjectDoubleHashMapSocialNetworkImpl<String>();
		try {
			ResultSet rs = stat.executeQuery("select * from boolean_how");
			
			while(rs.next()) {
				String res1 = rs.getString(1);
				String res2 = rs.getString(2);
				double value = rs.getDouble(3)/divisor;
				
				socialnetwork.addVertex(res1);
				socialnetwork.addVertex(res2);
				socialnetwork.addArc(res1,res2, value);
				if(maxValue < value) maxValue = value;
							
			} 
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
	
	public SocialNetwork<String> getNetwork(double sliderValue) {
		SocialNetwork<String> networkClone = new TObjectDoubleHashMapSocialNetworkImpl<String>();
		
		double percentage = 1.0 - sliderValue;
		double threshold = percentage * maxValue;
		
		Collection<String> vertices = socialnetwork.getVertices();
		
		for (String from : vertices) {
			Collection<String> outArcs = socialnetwork.getOutSet(from);
			
			for(String to : outArcs) {
				
				double value = socialnetwork.getArcValue(from, to);
				
				if(value >= threshold) {
					networkClone.addArc(from, to, value);
				}
			}
		} 
		
		return networkClone;	
	}
	
	private void countDivisor() {
		try {
			int totalTrace = 0; String caseColumnName = null;
			ResultSet rs = stat.executeQuery("show columns from " + parameter.getLogTable());
			if(rs.next()) {
				caseColumnName = rs.getString(1);
			}
			
			rs = stat.executeQuery("select count(*) from (select distinct " + caseColumnName + " from " + parameter.getLogTable() + ")");
			if(rs.next()) {
				totalTrace = rs.getInt(1);
			}
			
			divisor =  totalTrace;
			
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}
	
	public String getISInitialization() {
		return initializeType;
	}
}
