package org.processmining.indatabasepreprocessing.algorithms;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import javax.swing.JComponent;

import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.dataawarecnetminer.interactive.InteractiveDataAwareCausalMiner;
import org.processmining.dataawarecnetminer.interactive.InteractiveDataAwareCausalMinerController;
import org.processmining.dataawarecnetminer.model.DataRelationStorage;
import org.processmining.dataawarecnetminer.model.EventRelationStorage;
import org.processmining.indatabasepreprocessing.parameters.DBConnectionParameter;

import com.google.common.base.Stopwatch;
import com.google.common.collect.ImmutableSet;

public class InDBPreprocessingDHM implements InDBPreprocessingAlgorithm {

	private UIPluginContext context;
	private DBConnectionParameter parameter;
	private Statement stat;
	private XEventClasses eventClasses;
	
	public InDBPreprocessingDHM(UIPluginContext context, Connection conn, DBConnectionParameter parameter) {
		this.context = context;
		this.parameter = parameter;
		try {
			stat = conn.createStatement();
			stat.setFetchSize(100000);
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}

	public JComponent visualize() {
		long start =System.nanoTime();
		
		// retrieve and mine
		long startMining = System.nanoTime();
		InteractiveDataAwareCausalMiner causalMiner = mine();
		System.out.print("DHM," + String.format("%.3f", (System.nanoTime() - startMining) / 1000.0) + ";" );
		
		// visualization
		long startVisualization = System.nanoTime();
		InteractiveDataAwareCausalMinerController component = new InteractiveDataAwareCausalMinerController(context, causalMiner.getEventStorage(),
				causalMiner.getDataStorage());
		System.out.print(String.format("%.3f", (System.nanoTime() - startVisualization) / 1000.0) + ";" );
		
		System.out.print(String.format("%.3f", (System.nanoTime() - start) / 1000.0) + "\n" );
		
		return component;
	}
	
	private InteractiveDataAwareCausalMiner mine() {
		context.getProgress().setMaximum(3);
		
		Map<String, Class<?>> attributeTypes = new HashMap<String, Class<?>>();
		attributeTypes.put("concept:name", String.class);
		
		XEventClassifier classifier = new XEventNameClassifier();
		eventClasses = new XEventClasses(classifier);
		getEventClasses();

		context.log("Computing log relations ...");
		context.getProgress().setValue(1);

		Stopwatch watch = Stopwatch.createStarted();
		ExecutorService executorService = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
		try {
			EventRelationStorage eventRelationStorage = new EventRelationStorageImplDB(eventClasses, stat);
			DataRelationStorage dataRelations = DataRelationStorage.Factory.createDataRelations(eventRelationStorage,
					attributeTypes, ImmutableSet.<String>of(), executorService);
			
			/*System.out.println("PRINT RELATIONS");
			System.out.println(((EventRelationStorageImplDB) eventRelationStorage).printRelations());*/

			watch.stop();
			context.getProgress().setValue(2);
			context.log(MessageFormat.format("Finished in {0,number,#}ms", watch.elapsed(TimeUnit.MILLISECONDS)));

			context.getProgress().setValue(3);

			return new InteractiveDataAwareCausalMiner(eventRelationStorage, dataRelations);

		} finally {
			executorService.shutdown();
		}
	}
	
	private void getEventClasses() {
		
		ResultSet rs = null;
		try {
			rs = stat.executeQuery("select act from occurrence;");
			
			int i = 0;
			while(rs.next()) {				
				String act = rs.getString(1);
				eventClasses.register(act);
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
	
	
	

}
