package org.processmining.indatabasepreprocessing.algorithms;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import javax.swing.JComponent;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.classification.XEventNameClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryBufferedImpl;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.indatabasepreprocessing.parameters.DBConnectionParameter;
import org.processmining.models.heuristics.HeuristicsNet;
import org.processmining.models.heuristics.HeuristicsNetGraph;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.gui.HeuristicsNetVisualizer;
import org.processmining.plugins.heuristicsnet.miner.heuristics.miner.settings.HeuristicsMinerSettings;
import org.processmining.plugins.heuristicsnet.visualizer.annotatedvisualization.AnnotatedVisualizationGenerator;
import org.processmining.plugins.heuristicsnet.visualizer.annotatedvisualization.AnnotatedVisualizationSettings;

public class InDBPreprocessingFHM implements InDBPreprocessingAlgorithm {

	private UIPluginContext context;
	private DBConnectionParameter parameter;
	private Statement stat;
	private XLog log;
	
	public InDBPreprocessingFHM(UIPluginContext context, Connection conn, DBConnectionParameter parameter) {
		this.context = context;
		this.parameter = parameter;
		try {
			stat = conn.createStatement();
			stat.setFetchSize(100000);
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}

	public JComponent visualize() {
		long start = System.nanoTime();
		
		// import log
		long startImport = System.nanoTime();
		try {
			importLog();
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.out.print("FHM," + String.format("%.3f", (System.nanoTime() - startImport) / 1000.0) + ";" );
		
		// retrieve and mine
		long startMining = System.nanoTime();
		HeuristicsNet net = mine();
		System.out.print(String.format("%.3f", (System.nanoTime() - startMining) / 1000.0) + ";" );
		
		// visualize
		long startVisualization = System.nanoTime();
		AnnotatedVisualizationGenerator generator = new AnnotatedVisualizationGenerator();
		AnnotatedVisualizationSettings visualizationSettings = new AnnotatedVisualizationSettings();
		HeuristicsNetGraph graph = generator.generate(net, visualizationSettings);
		JComponent component = HeuristicsNetVisualizer.visualizeGraph(graph, net, visualizationSettings, context.getProgress());
		System.out.print(String.format("%.3f", (System.nanoTime() - startVisualization) / 1000.0) + ";" );
		
		System.out.print(String.format("%.3f", (System.nanoTime() - start) / 1000.0) + "\n" );
		
		return component;
	}
	
	private HeuristicsNet mine() {
		XEventClassifier classifier = new XEventNameClassifier();
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, classifier);
		
		HeuristicsMinerSettings settings = new HeuristicsMinerSettings();
		settings.setClassifier(classifier);
		
		FlexibleHeuristicsMinerDB fhm = new FlexibleHeuristicsMinerDB(context, log, logInfo, settings, stat);
		return fhm.mine();
	}
	
	private void importLog() throws Exception {
		
		XFactory factory = new XFactoryBufferedImpl();
		ResultSet rs = stat.executeQuery("select * from "+ parameter.getLogTable());
		HashMap<String, ArrayList<Event>> cases = new HashMap<String, ArrayList<Event>>();		
		
		while(rs.next()) {
			String caseid = rs.getString(1);
			String activity = rs.getString(2);
			Timestamp time = rs.getTimestamp(3);
			
			ArrayList<Event> events;
			if(cases.containsKey(caseid)) {
				events = cases.get(caseid);
			} else {
				events = new ArrayList<Event>();
			}			
			events.add(new Event(time.getTime(), activity));
			cases.put(caseid, events);
		}
		
		log = factory.createLog();
				
		for(String caseid : cases.keySet()) {
			XTrace trace = factory.createTrace();
			trace.getAttributes().put("concept:name", factory.createAttributeLiteral("concept:name", caseid, XConceptExtension.instance()));
			
			ArrayList<Event> events = cases.get(caseid);
			Collections.sort(events);
			int i;
			for(i = 0; i < events.size(); i++) {
				XEvent event = factory.createEvent();
				event.getAttributes().put("concept:name", factory.createAttributeLiteral("concept:name", events.get(i).label, XConceptExtension.instance()));
				event.getAttributes().put("time:timestamp", factory.createAttributeTimestamp("time:timestamp", events.get(i).time, XTimeExtension.instance()));
				trace.add(event);
			}
			
			log.add(trace);
		}		
	}

	private class Event implements Comparable<Event> {
		long time;
		String label;
		
		Event(long time, String label){
			this.time = time;
			this.label = label;
		}
		
		@Override
		public int compareTo(Event e2) {
			if (time - e2.time < 0){
				return -1;
			}else{
				if (time - e2.time > 0){
					return 1;
				}else{
					return 0;
				}
			}
		}
	}

}
