package org.processmining.indatabasepreprocessing.visualizers;

import java.awt.BorderLayout;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.indatabasepreprocessing.parameters.DBConnectionParameter;

@Plugin(name = "DB Connection Visualizer", parameterLabels = { "DB Connection Parameter" }, returnLabels = {"DB Connection Visualizer" }, returnTypes = { JComponent.class })
@Visualizer
public class DBConnectionVisualizer {
	
	private DBConnectionParameter parameter;
	private JComponent container;	
	private JTextArea textarea;
	
	private JComponent asComponent() {
		return container;
	}
	
	private void printParameter() {
		String text = "" +
				"Server: " + parameter.getServer() +
				"\n Database name: " + parameter.getDatabaseName() +
				"\n Log table: " + parameter.getLogTable() + 
				"\n Process mining algorithm: ";
		Object[] algo = parameter.getPMAlgorithms();
		for(int i = 0; i < algo.length; i++) {
			text += algo[i] + " ";
		}
		text += "" +
				"\n Involve existing data: " + parameter.getPreviousData();
		
		if(parameter.getRule() != null) {
			text += "" +
					"\n" + parameter.getRule();
		}
		
		if(parameter.getBehaviouralProfile() != null) {
			text += "" +
					"\n" + parameter.getBehaviouralProfile();
		}
		
		textarea.setText(text);		
	}
	
	private void setupLayout() {
		container.removeAll();
		container.setLayout(new BorderLayout());
		
		printParameter();
		container.add(textarea, BorderLayout.NORTH);	
		
		container.revalidate();
		container.repaint();		
	}
	
	
	@UITopiaVariant(affiliation = "Eindhoven University of Technology", author = "A. Syamsiyah", email = "A.Syamsiyah@tue.nl")
	@PluginVariant(variantLabel = "DB Connection Visualizer", requiredParameterLabels = { 0 })
	@Visualizer
	public JComponent start(PluginContext context, DBConnectionParameter parameter) {
		this.parameter = parameter;
		this.container = new JPanel();
		this.textarea = new JTextArea();
		
		setupLayout();
		return asComponent();		
	}
}
