package org.processmining.indatabasepreprocessing.visualizers;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.indatabasepreprocessing.algorithms.InDBPreprocessingController;
import org.processmining.indatabasepreprocessing.parameters.MiningAlgorithm;

@Plugin(name = "Mining with In Database Preprocessing Visualizer", parameterLabels = { "In DB Preprocessing Controller" }, returnLabels = {"In DB Preprocessing Visualizer" }, returnTypes = { JComponent.class })
@Visualizer
public class InDBPreprocessingVisualizer {
	
	private InDBPreprocessingController controller;
	private JComponent container;	
	private JPanel smallcontainer;
	private JButton refreshButton;
	private JComboBox<MiningAlgorithm> algorithmSelector;
	
	private JComponent asComponent() {
		return container;
	}
	
	private void setupLayout() {
		container.removeAll();
		
		smallcontainer.setLayout(new GridLayout(0,2));
		smallcontainer.add(algorithmSelector);	
		smallcontainer.add(refreshButton);
		
		container.setLayout(new BorderLayout());
		container.add(controller.visualize(), BorderLayout.CENTER);
		container.add(smallcontainer, BorderLayout.SOUTH);
		
		container.revalidate();
		container.repaint();		
	}
	
	
	@SuppressWarnings("unchecked")
	@UITopiaVariant(affiliation = "Eindhoven University of Technology", author = "A. Syamsiyah", email = "A.Syamsiyah@tue.nl")
	@PluginVariant(variantLabel = "Mining with In Database Preprocessing Visualizer", requiredParameterLabels = { 0 })
	@Visualizer
	public JComponent start(UIPluginContext context, InDBPreprocessingController controller) {
		this.controller = controller;
		container = new JPanel();
		smallcontainer = new JPanel();
		
		refreshButton = new JButton("Refresh"); 
		refreshButton.addActionListener(new RefreshButtonListener());
		
		algorithmSelector = new ProMComboBox<MiningAlgorithm>(controller.parameter.getPMAlgorithms());		
		algorithmSelector.setSelectedItem(controller.parameter.getPMAlgorithms()[0]);
		algorithmSelector.addItemListener(new AlgorithmChangeListener());
		
		setupLayout();
		return asComponent();		
	}
	
	private class RefreshButtonListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			setupLayout();
		}
		
	}
	
	private class AlgorithmChangeListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				MiningAlgorithm chosenalgorithm = (MiningAlgorithm) e.getItem();
				controller.setCurrentAlgorithm(chosenalgorithm);
				setupLayout();
			}
		}
	}
}
